/*
Copyright (C) 2005 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package traitement;

import java.awt.Image;
import java.awt.image.MemoryImageSource;
import java.io.*;
import java.net.URL;

public class ImageFITS implements ImageIntf {
    
    private Traitement app;
    private URL urlImage;
    private int dimx=0, dimy=0, bitpix=16;
    private int seuil_inf=0, seuil_sup=0;
    private Image img = null;
    private int[][] valeurs;
    
    public ImageFITS(URL urlImage, Traitement app) {
        this.urlImage = urlImage;
        this.app = app;
    }
    
    public String getNom() {
        return(new File(urlImage.getFile()).getName());
    }
    
    protected void lectureFITS() {
        BufferedInputStream in;
        try {
            in = new BufferedInputStream(urlImage.openStream());
        } catch(Exception ex) {
            System.err.println("Erreur  l'ouverture de " + urlImage.toExternalForm() + ": " + ex.getMessage());
            return;
        }
        boolean end_of_header = false;
        byte[] fits_bytes = new byte[80];
        byte[] fits_bytes2 = new byte[80];
        int bytesread, bytesread2;
        String fits_line;
        int pos = 0;
        
        // lecture de l'entte
        while (! end_of_header && (pos < 28800)) {
            try {
                if ((bytesread = in.read(fits_bytes, 0, 80)) != 80) {
                    if (bytesread == -1)
                        end_of_header = true;
                    else { // why do I need to do that ???
                        if ((bytesread2 = in.read(fits_bytes2, 0, 80-bytesread)) != 80-bytesread)
                            end_of_header = true;
                        else
                            System.arraycopy(fits_bytes2, 0, fits_bytes, bytesread, bytesread2);
                    }
                }
                pos += 80;
                fits_line = new String(fits_bytes);
                if (fits_line.startsWith("END")) {
                    if (pos/2880 != pos/2880.0) {
                        long toskip = (pos/2880+1)*2880 - pos;
                        while (toskip > 0) {
                            long skipped = in.skip(toskip);
                            toskip -= skipped;
                        }
                        pos = (pos/2880+1)*2880;
                    }
                    end_of_header = true;
                }
                int i = fits_line.indexOf('=');
                int j = fits_line.lastIndexOf('/');
                if (j == -1)
                    j = fits_line.length()-1;
                String tag;
                if (i != -1)
                    tag = fits_line.substring(0, i);
                else
                    tag = "";
                tag = tag.trim().toUpperCase();
                String value = fits_line.substring(i+1, j);
                value = value.trim();
                if (value.startsWith("'") && value.endsWith("'")) {
                    value = value.substring(1, value.length()-1);
                    value = value.trim();
                }
                String comment = fits_line.substring(j+1);
                comment = comment.trim(); /* comments are not used */
                
                //System.out.println(tag + " " + value + " " + comment); // for debug
                
                if ("NAXIS1".equals(tag))
                    dimx = Integer.valueOf(value).intValue();
                else if ("NAXIS2".equals(tag))
                    dimy = Integer.valueOf(value).intValue();
                else if ("NAXIS".equals(tag)) {
                    int naxis = Integer.valueOf(value).intValue();
                    if (naxis != 2)
                        System.err.println("erreur: nombre de dimensions non gr (NAXIS)");
                } else if ("BITPIX".equals(tag))
                    bitpix = Integer.valueOf(value).intValue();
                else if ("DATAMIN".equals(tag) && seuil_inf == 0)
                    seuil_inf = Integer.valueOf(value).intValue();
                else if ("DATAMAX".equals(tag) && seuil_sup == 0)
                    seuil_sup = Integer.valueOf(value).intValue();
                
            } catch (Exception ex) {
                System.err.println("Erreur durant la lecture de l'entte: " + ex.getClass().getName() + ": " + ex.getMessage());
                ex.printStackTrace();
                end_of_header = true;
            }
        }
        
        if (seuil_sup == 0)
            seuil_sup = 1000;
        
        // lecture de l'image
        valeurs = new int[dimx][dimy];
        int i=0, j=0, b1, b2, b3;
        int absbitpix = Math.abs(bitpix);
        try {
            if (absbitpix > 16)
                b3 = (in.read() << 8) | in.read();
            else
                b3 = 0;
            if (absbitpix > 8)
                b2 = in.read();
            else
                b2 = 0;
            b1 = in.read();
            while (b1 != -1) {
                if (bitpix > 0)
                    valeurs[i][j] = (b3 << 16) | (b2 << 8) | b1;
                else
                    // en esprant que les valeurs ne dpassent pas le max d'un entier,
                    // et en perdant les dcimales...
                    valeurs[i][j] = (int)Math.round(Float.intBitsToFloat((b3 << 16) | (b2 << 8) | b1));
                i++;
                if (i > dimx - 1) {
                    i = 0;
                    j++;
                }
                if (absbitpix > 16)
                    b3 = (in.read() << 8) | in.read();
                if (absbitpix > 8)
                    b2 = in.read();
                b1 = in.read();
            }
        } catch (Exception ex) {
            if (i != 0 || j != dimy) {
                System.err.println("Erreur  la lecture de " + urlImage.toExternalForm() + "  l'index ["+i+","+j+"]");
                System.err.println("(devrait tre ["+ 0 +","+ dimy +"])");
                ex.printStackTrace();
            }
        }
        
        try {
            in.close();
        } catch(Exception e) {
        }
    }
    
    // creation de l'objet Image en fonction des seuils
    private void creationImage() {
        int[][] palette = app.getPalette();
        int[] pix = new int[dimx * dimy];
        int v;
        int k=0;
        for (int j=0; j<dimy; j++) {
            for (int i=0; i<dimx; i++) {
                v = (int)Math.round( 256 * ((double)(valeurs[i][j] - seuil_inf)) / (seuil_sup - seuil_inf + 1) );
                if (v < 0)
                    v = 0;
                else if (v > 255)
                    v = 255;
                //pix[k++] = (255 << 24) | (v << 16) | (v << 8) | v;
                pix[k++] = (255 << 24) | (palette[v][0] << 16) | (palette[v][1] << 8) | palette[v][2];
            }
        }
        if (img != null)
            img.flush();
        img = app.createImage(new MemoryImageSource(dimx, dimy, pix, 0, dimx));
    }
    
    public Image getImage() {
        if (valeurs == null)
            lectureFITS();
        if (img == null && valeurs != null)
            creationImage();
        return(img);
    }
    
    public int getSeuilInf() {
        return(seuil_inf);
    }
    
    public int getSeuilSup() {
        return(seuil_sup);
    }
    
    public void setSeuils(int mini, int maxi) {
        seuil_inf = mini;
        seuil_sup = maxi;
        creationImage();
    }
    
    public void recreer() {
        if (img != null)
            creationImage();
    }
    
    public int[][] getValeurs() {
        if (valeurs == null)
            lectureFITS();
        return(valeurs);
    }
}
