/*

Copyright (C) 2012  Observatoire de Paris

The JavaScript code in this page is free software: you can
redistribute it and/or modify it under the terms of the GNU
General Public License (GNU GPL) as published by the Free Software
Foundation, either version 3 of the License, or (at your option)
any later version.  The code is distributed WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU GPL for more details.

As additional permission under GNU GPL version 3 section 7, you
may distribute non-source (e.g., minimized or compacted) forms of
that code without the copy of the GNU GPL normally required by
section 4, provided you include this license notice and a URL
through which recipients can access the Corresponding Source.

*/

function FenetreGraphe(vx, vy, labelx, labely) {
    this.vx = vx;
    this.vy = vy;
    this.labelx = labelx;
    this.labely = labely;
}

FenetreGraphe.prototype.afficher = function() {
    var div, divplot, plot, points, i, bfermer, somme, moyenne, ecart, divstat;
    
    div_overlay = document.getElementById('div-overlay-graphe');
    div_overlay.style.display = 'block';
    
    div = document.getElementById('div-graphe');
    if (div.firstChild == null) {
        divplot = document.createElement('div');
        divplot.setAttribute('id', 'plot');
        divplot.style.width = (div.offsetWidth - 10) + 'px';
        divplot.style.height = (div.offsetHeight - 100) + 'px';
        div.appendChild(divplot);
        
        
        somme = 0;
        for (i=0; i<this.vy.length; i++)
            somme += this.vy[i];
        moyenne = somme / this.vy.length;
        somme = 0;
        for (i=0; i<this.vy.length; i++)
            somme += Math.abs(this.vy[i] - moyenne);
        ecart = somme / this.vy.length;
        divstat = document.createElement('div');
        divstat.appendChild(document.createTextNode(
            Messages.get('moyenne') + ' ' + this.joliFormattage(moyenne, 4) +
            ' ; ' + Messages.get('écart-type') + ' ' + this.joliFormattage(ecart, 4)
        ));
        div.appendChild(divstat);
        
        
        div_boutons = document.createElement('div');
        div_boutons.setAttribute('id', 'div-boutons');
        
        bfermer = document.createElement('button');
        bfermer.setAttribute('id', 'fermer');
        bfermer.setAttribute('type', 'submit');
        bfermer.appendChild(document.createTextNode(Messages.get("Fermer")));
        bfermer.addEventListener('click', this.actionPerformed, false);
        div_boutons.appendChild(bfermer);
        
        div.appendChild(div_boutons);
    } else {
        divplot = document.getElementById('plot');
        $('#plot').empty();
    }
    
    points = [];
    for (i=0; i<this.vx.length; i++)
        if (this.vx[i] != null && this.vy[i] != null)
            points.push([this.vx[i], this.vy[i]]);
    
    $.jqplot('plot', [points], {
        sortData: false,
        axesDefaults: {
            labelRenderer: $.jqplot.CanvasAxisLabelRenderer,
            pad: 0
        },
        axes: {
            xaxis: {
                label: this.labelx
            },
            yaxis: {
                label: this.labely
            }
        },
        seriesDefaults: {
            lineWidth: 1.5,
            shadow: false
        },
        series : [
            {
                showLine: true,
                markerOptions: { show: false }
            }
        ],
        cursor: {
            show: true,
            zoom: true,
            showTooltip: true
        },
        highlighter: {
            show: true,
            useAxesFormatters: false
        }
    });

}

FenetreGraphe.prototype.actionPerformed = function(e) {
    div_overlay = document.getElementById('div-overlay-graphe');
    div_overlay.style.display = 'none';
}

FenetreGraphe.prototype.joliFormattage = function(d, precision) {
    if (isNaN(d))
        return('');
    if (d == 0)
        return('0');
    var facteur_precision = Math.pow(10, precision);
    var exposant = Math.floor(Math.log(Math.abs(d))/Math.log(10));
    var mantisse = d / (Math.pow(10,exposant));
    var s;
    if (exposant == 0) {
        mantisse = Math.round(mantisse*facteur_precision)/facteur_precision;
        s = ""+mantisse;
    } else if (exposant == 1) {
        mantisse = Math.round(mantisse*facteur_precision)/(facteur_precision/10);
        s = ""+mantisse;
    } else if (exposant == -1) {
        var signe;
        if (mantisse < 0)
            signe = "-";
        else
            signe = "";
        mantisse = Math.abs(Math.round(mantisse*facteur_precision));
        s = signe + "0."+Math.floor(mantisse);
        s = s.replace(/0+$/g, '');
    } else {
        mantisse = Math.round(mantisse*facteur_precision)/facteur_precision;
        s = mantisse + "E" + exposant;
    }
    return(s);
}
