/*

Copyright (C) 2012  Observatoire de Paris

The JavaScript code in this page is free software: you can
redistribute it and/or modify it under the terms of the GNU
General Public License (GNU GPL) as published by the Free Software
Foundation, either version 3 of the License, or (at your option)
any later version.  The code is distributed WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU GPL for more details.

As additional permission under GNU GPL version 3 section 7, you
may distribute non-source (e.g., minimized or compacted) forms of
that code without the copy of the GNU GPL normally required by
section 4, provided you include this license notice and a URL
through which recipients can access the Corresponding Source.

*/

function ImageFITS(urlImage, palette) {
    this.urlImage = urlImage;
    this.palette = palette;
    this.dimx = 0;
    this.dimy = 0;
    this.bitpix = 16;
    this.seuil_inf = null;
    this.seuil_sup = null;
    this.valeurs = null;
    this.canvas = document.createElement('canvas');
    this.canvas.setAttribute('width', this.dimx);
    this.canvas.setAttribute('height', this.dimy);
    this.ctx = this.canvas.getContext('2d');
    this.imagedata = null;
    this.ftexte = null;
    this.fpos = 0;
}

ImageFITS.prototype.lectureFichierBinaire = function(url) {
    var req;
    
    req = new XMLHttpRequest();
    req.open('GET', url, false);
    if (req.overrideMimeType) // pas dispo avec IE, même IE9
        req.overrideMimeType('text\/plain; charset=x-user-defined');
    req.send(null);
    this.fpos = 0;
    if (req.status != 200)
        this.ftexte = null;
    if (req.overrideMimeType)
        this.ftexte = req.responseText;
    else
        this.ftexte = (new VBArray(req.responseBody)).toArray(); // pour IE9
}

ImageFITS.prototype.octetSuivant = function() {
    var c, b;
    if (this.ftexte == null || this.ftexte.length <= this.fpos)
        return(-1);
    if (typeof this.ftexte == 'string') {
        c = this.ftexte.charCodeAt(this.fpos);
        b = c & 0xFF;
    } else {
        b = this.ftexte[this.fpos];
    }
    this.fpos ++;
    return(b);
}

ImageFITS.prototype.string80Suivant = function() {
    var s;
    
    if (typeof this.ftexte == 'string')
        s = this.ftexte.substring(this.fpos, this.fpos + 80);
    else
        s = String.fromCharCode.apply(String, this.ftexte.slice(this.fpos, this.fpos + 80));
    this.fpos += 80;
    return(s);
}

ImageFITS.prototype.conversionFloat = function(b0, b1, b2, b3) {
    var signe, exposant, mantisse;
    
    signe = 1 - (2 * (b0 >> 7)),
    exposant = (((b0 << 1) & 0xff) | (b1 >> 7)) - 127,
    mantisse = ((b1 & 0x7f) << 16) | (b2 << 8) | b3;
    
    if (exposant === 128) {
        if (mantisse !== 0)
            return(NaN);
        else
            return(signe * Infinity);
    }
    
    if (exposant === -127)
        return(signe * mantisse * Math.pow(2, -126 - 23));
    
    return(signe * (1 + mantisse * Math.pow(2, -23)) * Math.pow(2, exposant));
}

ImageFITS.prototype.sauter = function(n) {
    this.fpos += n;
}

ImageFITS.prototype.getNom = function() {
    var ind;
    ind = this.urlImage.lastIndexOf('/');
    if (ind == -1)
        return(this.urlImage);
    else
        return(this.urlImage.substring(ind + 1));
}

ImageFITS.prototype.trim = function(s) {
    return(s.replace(/^\s+|\s+$/g, ''));
}
ImageFITS.prototype.lectureFITS = function() {
    var end_of_header, fits_line, pos, toskip, skipped, i, j, tag, value,
        comment, naxis, b1, b2, b3, b4, absbitpix, datamin, datamax, v;
    
    this.lectureFichierBinaire(this.urlImage);
    if (this.ftexte == null) {
        alert("Erreur à l'ouverture de " + this.urlImage);
        return;
    }
    end_of_header = false;
    pos = 0;
    
    // lecture de l'entête
    while (! end_of_header && (pos < 28800)) {
        try {
            if (this.ftexte.length <= this.fpos + 80) {
                end_of_header = true;
                break;
            }
            fits_line = this.string80Suivant();
            pos += 80;
            if (fits_line.indexOf("END") === 0) {
                if (Math.floor(pos/2880) != pos/2880) {
                    toskip = (Math.floor(pos/2880) + 1) * 2880 - pos;
                    this.sauter(toskip);
                    pos = (pos/2880+1)*2880;
                }
                end_of_header = true;
            }
            i = fits_line.indexOf('=');
            j = fits_line.lastIndexOf('/');
            if (j == -1)
                j = fits_line.length - 1;
            if (i != -1)
                tag = fits_line.substring(0, i);
            else
                tag = '';
            tag = this.trim(tag).toUpperCase();
            value = fits_line.substring(i+1, j);
            value = this.trim(value);
            if (value.indexOf("'") === 0 && value.lastIndexOf("'") === value.length-1) {
                value = value.substring(1, value.length-1);
                value = this.trim(value);
            }
            comment = fits_line.substring(j+1);
            comment = this.trim(comment); /* comments are not used */
            
            if (tag == "NAXIS1")
                this.dimx = parseInt(value);
            else if (tag == "NAXIS2")
                this.dimy = parseInt(value);
            else if (tag == "NAXIS") {
                naxis = parseInt(value);
                if (naxis != 2)
                    alert("lecture du FITS: erreur: nombre de dimensions non géré (NAXIS)");
            } else if (tag == "BITPIX")
                this.bitpix = parseInt(value);
            else if (tag == "DATAMIN" && this.seuil_inf == null)
                this.seuil_inf = parseInt(value);
            else if (tag == "DATAMAX" && this.seuil_sup == null)
                this.seuil_sup = parseInt(value);
            
        } catch (ex) {
            alert("Erreur durant la lecture de l'entête de " + this.urlImage + ": " + ex);
            end_of_header = true;
        }
    }
    
    // lecture de l'image
    this.valeurs = Traitement.tableau2D(this.dimx, this.dimy);
    i = 0;
    j = 0;
    datamin = Number.MAX_VALUE;
    datamax = 0;
    absbitpix = Math.abs(this.bitpix);
    try {
        if (absbitpix > 16) {
            b4 = this.octetSuivant();
            b3 = this.octetSuivant();
        } else {
            b4 = 0;
            b3 = 0;
        }
        if (absbitpix > 8)
            b2 = this.octetSuivant();
        else
            b2 = 0;
        b1 = this.octetSuivant();
        while (b1 != -1 && j < this.dimy) {
            if (this.bitpix > 0)
                v = (b4 << 24) | (b3 << 16) | (b2 << 8) | b1;
            else
                // en espérant que les valeurs ne dépassent pas le max d'un entier,
                // et en perdant les décimales...
                v = Math.round(this.conversionFloat(b4, b3, b2, b1));
            this.valeurs[i][j] = v;
            if (v < datamin)
                datamin = v;
            if (v > datamax)
                datamax = v;
            i++;
            if (i > this.dimx - 1) {
                i = 0;
                j++;
            }
            if (absbitpix > 16) {
                b4 = this.octetSuivant();
                b3 = this.octetSuivant();
            } else {
                b4 = 0;
                b3 = 0;
            }
            if (absbitpix > 8)
                b2 = this.octetSuivant();
            else
                b2 = 0;
            b1 = this.octetSuivant();
        }
    } catch (ex) {
        if (i != 0 || j != this.dimy) {
            alert("Erreur à la lecture de " + this.urlImage + " à l'index ["+i+","+j+"]");
            alert("(devrait être ["+ 0 +","+ this.dimy +"])");
        }
    }
    if (this.seuil_inf == null)
        this.seuil_inf = datamin;
    if (this.seuil_sup == null)
        this.seuil_sup = datamax;
}

// creation de l'objet Image en fonction des seuils
ImageFITS.prototype.creationImage = function() {
    var v, index, j, i;
    
    index = 0;
    this.imagedata = this.ctx.createImageData(this.dimx, this.dimy);
    for (j=0; j<this.dimy; j++) {
        for (i=0; i<this.dimx; i++) {
            v = Math.round( 256 * (this.valeurs[i][j] - this.seuil_inf) / (this.seuil_sup - this.seuil_inf + 1) );
            if (v < 0)
                v = 0;
            else if (v > 255)
                v = 255;
            this.imagedata.data[index++] = this.palette[v][0];
            this.imagedata.data[index++] = this.palette[v][1];
            this.imagedata.data[index++] = this.palette[v][2];
            this.imagedata.data[index++] = 255;
        }
    }
}

ImageFITS.prototype.getImage = function() {
    if (this.valeurs == null)
        this.lectureFITS();
    if (this.imagedata == null && this.valeurs != null)
        this.creationImage();
    return(this.imagedata);
}

ImageFITS.prototype.getSeuilInf = function() {
    return(this.seuil_inf);
}

ImageFITS.prototype.getSeuilSup = function() {
    return(this.seuil_sup);
}

ImageFITS.prototype.setSeuils = function(mini, maxi) {
    this.seuil_inf = mini;
    this.seuil_sup = maxi;
    this.creationImage();
}

ImageFITS.prototype.setPalette = function(palette) {
    if (this.valeurs == null)
        this.lectureFITS();
    this.palette = palette;
    this.creationImage();
}

ImageFITS.prototype.recreer = function() {
    if (this.imagedata != null)
        this.creationImage();
}

ImageFITS.prototype.getValeurs = function() {
    if (this.valeurs == null)
        this.lectureFITS();
    return(this.valeurs);
}
