/*

Copyright (C) 2012  Observatoire de Paris

The JavaScript code in this page is free software: you can
redistribute it and/or modify it under the terms of the GNU
General Public License (GNU GPL) as published by the Free Software
Foundation, either version 3 of the License, or (at your option)
any later version.  The code is distributed WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU GPL for more details.

As additional permission under GNU GPL version 3 section 7, you
may distribute non-source (e.g., minimized or compacted) forms of
that code without the copy of the GNU GPL normally required by
section 4, provided you include this license notice and a URL
through which recipients can access the Corresponding Source.

*/

function ImageMemoire(nom, valeurs, palette) {
    this.seuil_inf = 0;
    this.seuil_sup = 1000;
    this.nom = nom;
    this.valeurs = valeurs;
    this.palette = palette;
    this.dimx = valeurs.length;
    this.dimy = valeurs[0].length;
    this.canvas = document.createElement('canvas');
    this.canvas.setAttribute('width', this.dimx);
    this.canvas.setAttribute('height', this.dimy);
    this.ctx = this.canvas.getContext('2d');
    this.imagedata = null;
}

ImageMemoire.prototype.getNom = function() {
    return(this.nom);
}

// creation de l'objet Image en fonction des seuils
ImageMemoire.prototype.creationImage = function() {
    var v, index, j, i;
    
    index = 0;
    this.imagedata = this.ctx.createImageData(this.dimx, this.dimy);
    for (j=0; j<this.dimy; j++) {
        for (i=0; i<this.dimx; i++) {
            v = Math.round( 256 * (this.valeurs[i][j] - this.seuil_inf) / (this.seuil_sup - this.seuil_inf + 1) );
            if (v < 0)
                v = 0;
            else if (v > 255)
                v = 255;
            if (this.palette[v] == undefined)
                alert("Erreur: mauvaise valeur de pixel: " + this.valeurs[i][j]);
            this.imagedata.data[index++] = this.palette[v][0];
            this.imagedata.data[index++] = this.palette[v][1];
            this.imagedata.data[index++] = this.palette[v][2];
            this.imagedata.data[index++] = 255;
        }
    }
}

ImageMemoire.prototype.getImage = function() {
    if (this.imagedata == null && this.valeurs != null)
        this.creationImage();
    return(this.imagedata);
}

ImageMemoire.prototype.getSeuilInf = function() {
    return(this.seuil_inf);
}

ImageMemoire.prototype.getSeuilSup = function() {
    return(this.seuil_sup);
}

ImageMemoire.prototype.setSeuils = function(mini, maxi) {
    this.seuil_inf = mini;
    this.seuil_sup = maxi;
    this.creationImage();
}

ImageMemoire.prototype.setPalette = function(palette) {
    this.palette = palette;
    this.creationImage();
}

ImageMemoire.prototype.recreer = function() {
    if (this.imagedata != null)
        this.creationImage();
}

ImageMemoire.prototype.getValeurs = function() {
    return(this.valeurs);
}
